__code__='GatU0D/[on&>$QoP+t8J.4qaV!^TmUms$:[XXXGniC.did2`dEJ1SaI8PFN?g"f:R;YUA-?i$Dkf9f^qO;+ZbE47o8n\'9/KbcGGD.":bEDppQCP/),m,Y@#YTR_4dDZThY$;c=U[$\\K2<<8mm"fqaU!?6Oo!2fgp!LnTE!P\\_!"F(dQ#iH-L;8?>M@t8WNhZFo)quQ:W^_;5G!D``f#!<!4(3#jANkDAIf"IEJiC*p)2&On124/rpJZ?WB.Yu^V^rl6e4=>6m/;HXO-Q\'3C:^gBd=:*1G.Q$U7HugQtMh*(I&Dmd2<hV`kYsqhA;(H7FeK8`k"RHITrh^#<@$a?J]b:Fq"^<F"WfGMF"gV@/*[t+I_8F4Q1.UPrK-^C!j"L-Koc>dCCGD&:5[n$P^fkU@[([X#7_)f\'m\'>>T4]]+Bc(2dg0$;*fm?(P\\N3JbX3++*B9-(\'ie\\NojgGdj$GDm"n`D1&^UY\'\\.X(dO[Cr][k)>c0fCb;)b<m6A+%`qqhT*-D#qmP`q7Oa@-IV"4>R]-4-M>WVYqW2!#((K4H]/(:k-$m_Jb#3)i?r\\h+:BiR016?JiWl-<MR3jhcQ0I!D]a\\b`5<>A&mHq?*RrP[d%b"9Zk+6pkC"%-C.u;5sNRC4l>,k.0C%<+<g!5%j$qmJ*gsgD=F]5E_R4X8CdAh!C*igr.0:DF=peTgP8fQo-do:\'NYkFj.P%W)-&#_bg$m:U\\fj&T7&fX[84t3m\'9A.Vo-aU&f2Zfc.\\bT.L*iA,..p&q$\\9[beY;`Zb<ZF7WZhp+S9!$H4&EcFtT68CI>!sT!rrgR,]HWPjOi9)"g%EPt([6Y]2ksosb]nU#/NUQ8(<S^D0KG`V^$eSnj"7ML\\J@T<_SX,rVR]2?h(6:RVp=`rKju.V*VhNaA!^<L>.FLqAZrU;@^<egs%6,J3XK_0)kadbqZ/R,2=:^QS$:lj!aGD&%-,)!U;S_mXmc#[ctU!.T6tKQ2qVc?UVra4gR;Ws74h-ah"+LuL3u4AA&=#,cS-HmPTm.t`j+3=Ei\\Z-c[THL]<M=ZHc[l5CG*0O5Lu2;e"Z@srtCYXdAVEm\\?LCk~>'




"encrypt example 1 to demonstrate the encryption feature"

def test(filename="example_1.rml"):
    from rlextra.utils import pdfencrypt
    from rlextra.rml2pdf import rml2pdf
    encryption = pdfencrypt.StandardEncryption("User", None, canPrint=1, canModify=0, canCopy=0, canAnnotate=0)
    text = open(filename).read()
    result = rml2pdf.go(text, encryption=encryption)
    print(result.filename, "encrypted with user password User and no Owner password")

if __name__=="__main__":
    import sys
    try:
        filename = sys.argv[1]
    except:
        print("processing example_1.rml to process another file provide a filename argument")
        test()
    else:
        test(filename)

        
    
	


"""
This script enumerates all loaded modules, creates a pythonxx.zip for each
and copyies associated .pyds. The aim is to prepare a package of modules
you will need to distribute.

Usage: When your program ends, add these two lines:

import dumpmods
dumpmods.analyze_modules(<path to store files in>)

where <path to store files in> has obvious connotations.
"""
import sys
import os
import shutil
import zipfile

def getarcname(filename):
    lfn=filename.lower()
    for p in [p for p in sys.path if p.endswith('site-packages')]+[p for p in sys.path if not p.endswith('site-packages')]:
        if lfn.startswith(p.lower()):
            return filename[len(p)+1:]
    return filename

def analyze_modules(targetdir):
    if os.path.isfile(targetdir):
        print("Cannot use %s as a target dir" % targetdir)
    else:
        try:
            os.makedirs(targetdir)
        except IOError:
            pass
    print("Analyzing modules needed...")

    zname = os.path.join(targetdir, "python%s.zip" % ''.join(sys.version.split()[0].split('.')[:2]))
    print("Creating %s..." % zname)
    z = zipfile.ZipFile(zname, "w", zipfile.ZIP_DEFLATED)

    encpath = None

    for modulename in list(sys.modules.keys()):
        module = sys.modules[modulename]
        try:
            filename = module.__file__
        except:
            if repr(module).find("(built-in)") < 0:
                print("---> IGNORING %s (%s)" % (modulename, module))
            continue

        # windows:
        #filename = filename.lower()
        if filename.lower().endswith(".pyc") or filename.lower().endswith(".py"):
            arcname = getarcname(filename)
            if arcname.startswith("encodings\\"):
                print("skipping encodings, will be filled in later...")
                if not encpath:
                    encpath = os.path.dirname(filename)
            else:
                print("adding %s as %s" % (filename, arcname))
                z.write(filename, arcname)
        elif filename.endswith(".pyd"):
            nname = os.path.join(targetdir, os.path.basename(filename))
            print("copying %s as %s" % (filename, nname))
            shutil.copy( filename, nname )

    # add encodings
    if encpath:
        F = os.listdir(encpath)
        FC = [x for x in F if x.lower().endswith('.pyc')]
        F = list(filter(lambda x,FC=FC: x.lower().endswith('.py') and x+'c' not in FC,F))
        for f in FC+F:
            source = os.path.join(encpath, f)
            target = "encodings\\%s" % f
            print("adding %s as %s" % (source, target))
            z.write(source, target)

    z.close()

if __name__ == "__main__":
    analyze_modules("z:\\")



__code__='Gas1]gMYJ*&Daab91rsI;YgU\'\\>_d2l%N/g&-C:3OO?r_/O]kS3SnUglIbOmVG42d.=L;"i/qC;.t>AK1JU9S23*a;]/+)3<cdsR)>R"Cq\\3?S!&.lGQ\\855SJ.mn1n/O>,<"pQk8Mg;_/_*Q$EV=gfJpL]9\'e=&@$@\\9VV(O=aj9f"Z:[=)][pu;cLF#pgu,`7:<DJ[%XD^"bHWR6k+(\'-G-smkZPAqW*H/;%I9=]E!/JKhO9TDBq)Pj*7Nj<1-u:e(I1TlshAXD9?k`0scB9thO]-;#6Q<UA<&Cq(/4O?O>&7c^[&<RFKslQaBI&q7gX=d-I+?kQ>CEbVdF5rR=0>FF/]`L*MhN7QMNJg?#J2Do;nXdWConG$-)([u^X%aeGEm.9/Qp9J_l=DWFg+d2-pNgD:+H[=\'q_:**+KWp_[%jB&j2#UA&WGA2BYH9EC..Z*@DkOS9Dh3!SSj?3UF7AIn,GbBU3iF`XqX$1h9*)dd0%u2`H"%#uu1Rk!jd2C&68"c+s\'u-oj(n!D,Cb7_9moqb"hgWn$.i@nU?jJ_RKCW,7rZ[<CU8*EW&6BL]l"h:NN/j$iL[Y8io;r9%gW&=lo41]H.X*.%V5j5t9N*-!.,%M_F0i\'=K]l,H8F"/\\;=(2HdG>j2#nBC:gm/&5r?*T?CtP$@Xuir$YUr)XZ"n#n9*8>\\%L?\\OAqcjc."IN-lkBJEkBd\'tUNJXMb9II8-NiSO\'\'P3Lrq$kN!=9P/_p)1%daXrhVKN"9!bK4MR8]oMU4M>>/PJun.XUocjs2?=_=gApGI/]2_Lof?u.!3A*j[UF1CD%4]-maTkH]3]i8k%j%(o2DeO:\\g^ND?4j_/+Sgsrm+DM6tHE&JSCeWi_Eik37lh3ic$^2;_Z\':?@c%4pHeP_3DT0@VRUE/`lJdZW$p[8:ar7Fa)Q_33&4).$.GEJG[%\';+JhR0op\'h&9d=BBEF2^A0O3JRMhFjdo@0GeJ"@O[9F^@o+(JQr~>'



# simple program to generate a PDF document. data comes from a preppy file and is generated into a PDF
import os
import preppy
from rlextra.rml2pdf import rml2pdf
from rlextra.radxml.xmlutils import xml2doctree


def generatePDF(xml='''<doc><a>AAAAA</a></doc>'''):
    # 1 - get the prep file
    module = preppy.getModule('travel_template.prep')
    preppydict =  dict( doc = xml2doctree(xml),)  #this is the preppy dictionary
    #print "doc: %s" % doc
    rml = module.getOutput(preppydict)
    outputFileName='myPDFfile.pdf'
    saveRml='latest.rml'
    rml2pdf.go(rml, outputFileName=outputFileName, outDir='.',saveRml=saveRml)
    print("generated %s and %s" % (outputFileName, saveRml))

def parseCommmandLine():
    from optparse import OptionParser
    parser = OptionParser('test [options]')
    parser.set_defaults(dirName=os.getcwd())
    parser.add_option("--xml", help='retrieve data from an XML document') 
    return parser.parse_args()

def main():
    options, args = parseCommmandLine()

    # check whether we have a xml document for reading      
    if options.xml:
        xmlFile = options.xml
        f = open(xmlFile, 'r')
        xmlDoc = f.read()
        f.close()
    generatePDF(xml=xmlDoc)
    
if __name__=="__main__":
    main()




#!/usr/local/bin/python
"""Script to check for bad UTF-8 in a MySQL database.
NOT dependent on any assumptions about Django.

Tested on:
 - MySQL 5.0
"""
from __future__ import print_function
import sys

def getConnection(db='',host='',user='',passwd=''):
    # local
    import MySQLdb
    conn = MySQLdb.connect(
        host=host,
        user=user,
        passwd=passwd,
        db=db,
        )
    return conn

def parseCommandLine():
    """Examines options and does preliminary checking"""
    from optparse import OptionParser

    parser = OptionParser(usage="%prog [options] database....",version='$LastChangedRevision$'.split(':')[1].strip('$').strip())
    parser.add_option("-v", "--verbose",
                      action="count", dest="verbose", default=0,
                      help="Print lots of information about what is going on")
    parser.add_option("--host",
                      dest="host",
                      default='localhost',
                      help="hostname (localhost)")
    parser.add_option("-u","--user",
                      dest="user",
                      default='root',
                      help="user name (root) for the db")
    parser.add_option("-p","--password",
                      dest="passwd",
                      default='',
                      help="password for the db")
    parser.add_option("--utf8",
                      action="store_true", dest="utf8", default=False,
                      help="checks given field(s) are in utf8")
    parser.add_option("--table",
                      dest="table",
                      default='',
                      help="only check specified table")
    parser.add_option("--field",
                      dest="field",
                      default='',
                      help="only check specified field, a table must also be specified")
    parser.add_option("--wellformed",
                      action="store_true", dest="wellformed", default=False,
                      help="check if xml is well formed")
    parser.add_option("--tags",
                      action="store_true", dest="tags", default=False,
                      help="shows different tags and attributes encountered")
    parser.add_option("--test",
                      action="store_true",
                      dest="test",
                      default=False,
                      help="Run unit tests")

    """options for
  
    --tags:  will run records
        through a forgiving html parser and show all different tags and attributes encountered

    #limiting output
    --max=10  just an idea, limit to this many records?
    other common problems:  entities (which ones); double-escaping; unescaped special characters

    --show  (duplicates verbose?) actually shows you the fields, rather than the content.    
    """
    options,args = parser.parse_args()
    if not options.test:
        if not len(args):
            parser.error("%prog needs at least one database argument")
    return options,args

def getSchema(options,conn, db):
    #Gets all columnRows
    cur = conn.cursor()
    try:
        cur.execute("""\
        SELECT t.TABLE_NAME, c.COLUMN_NAME, c.COLUMN_TYPE FROM information_schema.TABLES t, information_schema.COLUMNS c
        where t.TABLE_SCHEMA = c.TABLE_SCHEMA
        and t.TABLE_NAME = c.TABLE_NAME
        and (c.COLUMN_TYPE like '%%char%%' or c.COLUMN_TYPE like '%%text%%')
        and t.TABLE_SCHEMA = %s
        order by t.TABLE_NAME, c.COLUMN_NAME, c.COLUMN_TYPE""",(db,),)
        columnRows = cur.fetchall()
        return columnRows
    except:
        try:
            import re
            pat=re.compile('^.*(?:text|char).*$',re.I)
            cur.execute("""show table status from `%s`""" % db)
            T = [t[0] for t in cur.fetchall()]
            columnRows = []
            if options.table:
                tableName = options.table
                cur.execute("""show columns from `%s`.`%s`""" % (db,tableName))
                data = cur.fetchall()
                columnRows += [(tableName,x[0],x[1]) for x in data if pat.match(x[1])]
                return columnRows
            else:
                for tableName in T:
                    cur.execute("""show columns from `%s`.`%s`""" % (db,tableName))
                    data = cur.fetchall()
                    columnRows += [(tableName,x[0],x[1]) for x in data if pat.match(x[1])]
                return columnRows
        except:
            print("""Could not get schema as per modern MySQL and older version failed as well""", file=sys.stderr)
            raise

def getTable(conn, db, tableName):
    sql = 'SELECT * FROM `%s`.`%s`' % (db, tableName)
    cur = conn.cursor()
    cur.execute(sql)
    data = cur.fetchall()
    return data

def getField(conn, db, tableName, colName):
    sql = 'SELECT `%s` FROM `%s`.`%s`' % (colName, db, tableName)
    cur = conn.cursor()
    cur.execute(sql)
    data = cur.fetchall()
    return data

def checkUTF8(data):
    """Verifies that data is proper UTF8
    """
    try:
        str(data).decode('utf8')
    except UnicodeDecodeError:
        print('Failed to decode as UTF8:\n%s\n' % str(data))

def checkWellFormed(data):
    """Checks content is valid XML.
    >>> 2 + 2
    4
    >>> 2 + 3
    6
    >>> data = [["<p>blah blah >/p>"]]
    >>> checkWellFormed(data)
    0
    """    
    import pyRXP
    parser = pyRXP.Parser(ExpandCharacterEntities = 0,
                          ExpandGeneralEntities = 0,
                          XMLPredefinedEntities = 1,
                          ErrorOnBadCharacterEntities = 0,
                          ErrorOnUndefinedEntities = 0,
                          CaseInsensitive = 0,
                          IgnoreEntities = 1,
                          ReturnList=1,
                          AllowMultipleElements = 1,
                          )
    if str(data).find('<')>-1 or str(data).find('>')>-1:
        try:
            parser('<fragment>%s</fragment>' % str(data))
        except pyRXP.error as e:
            print(e.args[0]) #first error message
            print(str(data)+'\n')

def findTags(t,tags=None):
    if tags is None: tags = []
    tag = t[0]
    if tag not in tags:
        tags.append(tag)
    if t[2]:
        for c in t[2]:
            if isinstance(c,tuple):
                findTags(c,tags)
    return tags

def checkTags(data, tags=None):
    if tags == None:
        tags = []
    import pyRXP
    parser = pyRXP.Parser(ExpandCharacterEntities = 0,
                          ExpandGeneralEntities = 0,
                          XMLPredefinedEntities = 1,
                          ErrorOnBadCharacterEntities = 0,
                          ErrorOnUndefinedEntities = 0,
                          CaseInsensitive = 0,
                          IgnoreEntities = 1,
                          ReturnList=1,
                          AllowMultipleElements = 0,
                          )
    if str(data).find('<')>-1 or str(data).find('>')>-1:
        try:
            d = parser('<fragment>%s</fragment>' % str(data))
            newTags = findTags(d[0])
            for t in newTags:
                if t not in tags:
                    tags.append(t)
            tags.remove('fragment')
            return tags
        except pyRXP.error:
            pass
        
def checkDataSet(dataset, checkFunc, rtrn=None):
    """Applies checkFunc to each cell of each row of dataset.
    Handles errors and tells you what row they occurred in.
    Can also return a result"""
    collection = []
    for d in dataset:
        for i in d:
            if rtrn:
                result = checkFunc(i)
                if result:
                    for a in result:
                        if a not in collection:
                            collection.append(a)
            else:
                checkFunc(i)
    if rtrn: return collection
                
def checkdb(options,db):
    conn = getConnection(db=db,host=options.host,user=options.user,passwd=options.passwd)
    verbose = options.verbose

    if options.field and not options.table:
        print('You have to specify which table %s is in' % options.field)
        sys.exit()

    #optionsDict = {'utf8': checkUTF8, 'wellformed': checkWellFormed, 'tags': checkTags,}
    if options.table:
        if options.field:
            dataset = getField(conn, db, options.table, options.field)
        else:
            dataset = getTable(conn, db, options.table)

        print('***checking table %s***\n' % options.table)
        if options.utf8:
            checkDataSet(dataset, checkUTF8)
        if options.wellformed:
            checkDataSet(dataset, checkWellFormed)
        if options.tags:
            print(checkDataSet(dataset, checkTags, True))
        print('***Done checking table %s***' % options.table)
        
    else:
        columnRows = getSchema(options,conn, db)        
        if verbose:
            print(db)
        if verbose>2:
            print(columnRows)
        cur = conn.cursor()
        tags = []
        for tableName, colName, colType in columnRows:
            if verbose>1:
                print(tableName, colName, colType)
            cur.execute("SELECT `%s` FROM `%s`.`%s`" % (colName, db, tableName))
            dataset = cur.fetchall()
            if options.utf8:
                print('Checking column %s in table %s...' % (colName, tableName))
                checkDataSet(dataset, checkUTF8)
                print('Done checking column %s in table %s...\n' % (colName, tableName))
            if options.wellformed:
                print('Checking column %s in table %s...' % (colName, tableName))
                checkDataSet(dataset, checkWellFormed)
                print('Done checking column %s in table %s...\n' % (colName, tableName))
            if options.tags:
                newTags =  checkDataSet(dataset, checkTags, True)
                if newTags:
                    for t in newTags:
                        if t not in tags:
                            tags.append(t)
        if options.tags:
            print(tags)
        
def main():
    options,args = parseCommandLine()
    if options.test:
        import doctest, rlextra.utils.dbcheck
        doctest.testmod(rlextra.utils.dbcheck)
    else:
        for db in args:
            checkdb(options,db)


if __name__ == '__main__':
    main()



__code__='Gas1]9lJKG&Daab%mjtt$,(m@TVt>sJH?Q/KRgEPUa/nK&7e%lleT"HNbrquD/,>7HKBqOQ[In%E+6gYg<]n@p1_/qo4DVmcTfP$miniGY[e$T/j&,/@?cm!_T5Pp.&TGld(p*m0obGA?j0SA-!"X@=G<;Y1[G14ATOrq/q0GiU(>?qq-<fm3JLJj78CD0$U^JJYH(t"dJ-gOoBam;ED!=`+kAtL]DhG"oZ9rG!\'$3S`6k5\'\'L&7Oc6sqQ"9iGmf>fnkGVq0n"^@XTFFehg\'fdOp3n&hp>ZN\\jFZbQ0%^j7#]l^1^.>:6Bf4_)Fh*pS2DnKu&?!DaQf$^nQ2HQJJ+tLd?;+O(d.Efd@aht6>7[`+sgQ&][r?.gCeUK0-O:RTFDVqcBfT5;qq"=/P]s!#af)(qVaR%t1dujj\'OI/LNREiU3)I5Er/D>gkA_2pS*d?GAijugjog?*!>k`$k$V)78J^h@#\'V%P+9@AnBThNq$\\\\XJ3W2@%<XuG6\\bWqVg)$W9c,\\L^5JDXF?]JUG5UK$fo?JjYt[me"KHCM>bpM;XnfClk]n#7FibZL6[O9>Vur&?_3P65Sdr)+aTU7\\AU`ICCHPK85C>X-c\\*Z#I8U#O]=*9\\+??(3n5R->%,E[-U/#@3sHQHh=&!O%p`KtlY>DEG2JGOF==s2s.&pK-cG9]o2l?Up!3a:$Id5nE]bBX(lbgpe`VZ\'eYk*U`A#dIuX%deV#X9J1EaVR>G7_e=,7$UW.eb%*Hf:$4Unmu\'FOR.#Yeb^6e\\*;t/\\"41pH#/CK`3OTK\'9m!N?[+YfS?ut^&"3^sqRVVELCqN1-%<B-H<[s,1EMBr^!/%>0I%_tdDl>:_Ep\\Fd1_>,H444q*F?YsLf.?D+9J>[3i.+ufUOuJiS/\'jbCfdj"OVaU_U5Is>B[1.RZ]aX#@0fgmZ[obl+],g!1V0(\'4b3s`PQ%GBXScV)X/D=;DrIUIc5)>fT\\=meRT?8>p)>M1>]B~>'



from __future__ import print_function
"""structured text for general xml

This module encodes xml markup as "more easily editable" structured text,
and provides the structured text --> xml conversion also.

THIS IS "QUICK AND DIRTY" AND IS NOT INTENDED FOR
USE IN "BOUNDARY CASES" -- TO HANDLE BOUNDARY CASES PLEASE
EDIT XML DIRECTLY RATHER THAN USE THIS CONVENIENCE.

It is expected that this approach is "good enough" for
most XML usage -- but for some uses it will break.
See assumptions below.

Conventions:

The XML

# text before <tag a="b">some text</tag> text after

becomes (inline structured text convention)

# text before .tag(a="b")some text\n
# text after

where the parens can be omitted when there are no attributes.
OR for longer text (indentation structured text convention)

# text before .tag(a="b")\n
#      longer\n
#      text\n
# text after

Tag names may be abbreviated.  If "tag" is abbreviated to "t"
then 

# text before <tag>some text</tag> text after

may be written (inline convention)

# text before .t some text\n
# text after

OR (indentation convention)

# text before .t
#     some\n
#     text\n
# text after

OR (equivalently, but maybe more readable)

# text before 
# .t
#     some\n
#     text\n
# text after

Convention for tags with no content:

# text before <tag a="b"/> text after

becomes

# text before .tag(a="b")! text after

or if no attributes and abbreviated

# text before .t! text after

OR if it has been declared converter.contentless("t") simply

# text before .t text after

If you need to make a string of the form "z.strip" without intending
it to indicate markup, spell it "z\.strip".

special case: <b>xxx</b> becomes **xxx** and <i>xxx</i> becomes *xxx*

The following assumptions are assumed by the processor implementation.

ASSUMPTIONS: 
    1) all tags are surrounded by whitespace.
    2) all white space (except newlines) is "the same."
        (no preformatted text, for example -- but preserve newlines!)
    3) indentations are strictly consistent (no replacing of spaces with tabs)
    4) strings of form .X* (where X is alphabetic) never occur in the intended
        XML text output.
    5) double quotes and parentheses and never appear in attribute values
    6) escaped CDATA text is NOT PERMITTED.
    7) nesting of <b> and <i> is NOT PERMITTED.

In the event that tags contain tags indentation conventions are nested.

"""
from reportlab.lib.utils import ascii_letters
import string
class XMLTextConverter:
    def __init__(self, indent=" "*4):
        self.indent = indent
        self.TagToAbbreviation = {}
        self.AbbreviationToTag = {}
        self.contentlessTags = {}
    def contentless(self, tagname):
        self.contentlessTags[tagname] = tagname
    def abbreviate(self, tagname, abbreviation):
        a2t = self.AbbreviationToTag
        t2a = self.TagToAbbreviation
        if not tagname or not abbreviation:
            raise ValueError("cannot abbreviate using the empty string " + repr((abbreviation, tagname)))
        if abbreviation in a2t and a2t[abbreviation]!=tagname:
            raise ValueError("abbreviation in use %s (%s)" % (repr(abbreviation), repr(tagname)))
        # it doesn't matter if tags have multiple abbreviations
        t2a[tagname] = abbreviation
        a2t[abbreviation] = tagname
    def text2XML(self, text, start=0, end=None, level=0):
        # start must point to beginning of line whenever not in the middle of nonwhite line segment
        # the text[start:(end)] must all be at or past the indentation level of the first line.
        levelindent = "  "*level
        resultlist = [] # list of string
        if end is None:
            end = len(text)
        #print levelindent, "===text2xml==="
        #print indentText( text[start:end], levelindent )
        cursor = start
        alphanum = ascii_letters + string.digits
        while cursor<end:
            #print levelindent, "cursor is ", cursor, "near", repr(text[cursor: cursor+10])
            foundtag = None
            beforetag = cursor
            # look for first "." followed by an alpha
            while foundtag is None and cursor<end:
                firstmark = text.find(".", cursor, end)
                #print levelindent, "...cursor is", cursor, "firstmark", firstmark
                if firstmark>=cursor:
                    aftercursor = firstmark+1
                    next = text[aftercursor:aftercursor+1]
                    #print levelindent, "...next", next
                    # only recognize .alpha but not \.alpha
                    if next in ascii_letters and text[firstmark-1:firstmark]!='\\':
                        foundtag = cursor = aftercursor
                    else:
                        cursor = aftercursor
                else:
                    cursor = end
            if foundtag is None:
                # just indent the tagless text
                remainder = text[beforetag:end].strip()
                if "\n" in remainder:
                    remainder = "\n"+indentText(remainder, levelindent)
                #print levelindent, "finishing with remainder", repr(remainder)
                resultlist.append( xmlEscape(remainder) )
                cursor = end
            else: # foundtag is not None: process the tag and content if present
                # find the previous newline
                taglinestart = text.rfind("\n", start, foundtag)
                if taglinestart<start:
                    taglinestart=start
                else:
                    taglinestart = taglinestart+1
                # look for first nonalphanumeric
                cursor = aftercursor
                foundtagnameend = None
                while cursor<end and foundtagnameend is None:
                    aftercursor = cursor+1
                    nextchar = text[cursor:aftercursor]
                    if nextchar not in alphanum:
                        foundtagnameend = aftercursor
                    else:
                        cursor = aftercursor
                if cursor==end:
                    foundtagnameend = end
                tagname = text[foundtag:cursor]
                #print levelindent, "tagname", repr(tagname)
                tagname = self.AbbreviationToTag.get(tagname, tagname)
                #print levelindent, "unabbreviated tagname", repr(tagname)
                textbeforetag = text[beforetag:foundtag-1]
                aftercursor = cursor+1
                nextchar = text[cursor:aftercursor]
                #print levelindent, "nextchar", nextchar
                tagattributesstring = None
                closeparenlocation = cursor-1 # end of tag name is end of attributes by default
                if nextchar=="(":
                    #print levelindent, "tag has attributes, look for next close paren"
                    # look for close paren; make sure the number of quotes inside is even
                    seek = cursor
                    closeparenok = None
                    while not closeparenok and seek<end:
                        closeparenlocation = text.find(")", seek, end)
                        if closeparenlocation<seek:
                            raise ValueError("found '(' after %s tagname %s but no unquoted ')' follows before %s near %s" % (
                                cursor, repr(tagname), end, repr(text[seek:seek+20])))
                        # check that the quotesplits are odd (ie the number of quotes are even)
                        segment = text[cursor:closeparenlocation]
                        #singlequotesplit = segment.split("'")
                        doublequotesplit = segment.split('"')
                        #print doublequotesplit
                        if len(doublequotesplit)%2==1:
                            closeparenok = 1
                        seek = closeparenlocation+1
                    # note: it is legal to have newlines between the parens
                    tagattributesstring = text[aftercursor:closeparenlocation]
                    cursor = closeparenlocation+1
                    # check validity of the attribute string
                    tagattributesstringcursor = 0
                    equallocation = tagattributesstring.find("=", tagattributesstringcursor)
                    while equallocation>=tagattributesstringcursor:
                        attributename = tagattributesstring[tagattributesstringcursor:equallocation]
                        attributename = attributename.strip()
                        #print levelindent, "attributename", attributename
                        afterequal = equallocation+1
                        afterquote = afterequal+1
                        quotechar = tagattributesstring[afterequal:afterquote]
                        #print levelindent, "quotechar", quotechar
                        if quotechar not in ('"', "'"):
                            raise ValueError("after '%s=' a quote character is required in %s" % (
                                attributename, repr(tagattributesstring)))
                        # search for matching quote
                        endquotelocation = tagattributesstring.find(quotechar, afterquote)
                        if endquotelocation<afterquote:
                            raise ValueError("after %s=%s... a matching close quote is required in %s" % (
                                attributename, quotechar, repr(tagattributesstring)))
                        #attributevalue = tagattributesstring[afterquote:endquotelocation]
                        #print levelindent, ".....attribute", repr(attributename), "==", repr(attributevalue)
                        equallocation = tagattributesstring.find("=", endquotelocation)
                        tagattributesstringcursor = endquotelocation+1
                # emit the text before the tag
                textbeforetag = indentText(textbeforetag, levelindent)
                #print levelindent, "textbeforetag is", repr(textbeforetag), "tag is", tagname, "cursor at", repr(text[cursor:cursor+1])
                resultlist.append( xmlEscape(textbeforetag) )
                # emit the start of start tag
                resultlist.append("\n%s <%s" % (levelindent, tagname))
                # emit the attributes XXXX DOES THIS NEED TO BE ESCAPED?
                if tagattributesstring:
                    #print levelindent, "tag attribute string is", repr(tagattributesstring)
                    tagattributesstring = xmlEscape(tagattributesstring) # DOES NOT ESCAPE DOUBLE QUOTES!!
                    resultlist.append(" %s" % tagattributesstring)
                # figure out if there should be content
                cursor = closeparenlocation+1
                aftercursor = cursor+1
                aftercloseparenchar = text[cursor:aftercursor]
                nocontent = None
                if aftercloseparenchar == "!":
                    nocontent = 1
                    cursor = aftercursor
                if tagname in self.contentlessTags:
                    nocontent = 1
                if nocontent:
                    # close the tag and proceed
                    resultlist.append("/> ")
                    #cursor = aftercursor
                else:
                    # close the start tag
                    resultlist.append(">")
                    # find the end of the content
                    # if it is inline the next newline is the end
                    inlinestyle = None
                    nextnewline = text.find("\n", cursor, end)
                    # if no newline then inline
                    if nextnewline<cursor:
                        #print levelindent, "tag is inline because no newline found in region"
                        inlinestyle = 1
                        contentend = end
                    # if newline found and substring to newline is not white then inline
                    elif text[cursor:nextnewline].strip():
                        #print levelindent, "tag is inline because cursor to newline is nonwhite", repr(text[cursor:nextnewline])
                        inlinestyle = 1
                        contentend = nextnewline+1
                    else:
                        # move cursor past the newline
                        cursor = nextnewline+1
                    if inlinestyle:
                        xmlcontent = self.text2XML(text, start=cursor, end=contentend, level=level+1)
                        cursor = contentend
                    else:
                        # otherwise it must be indentation based: find next dedent (no dedent means "" content)
                        # first determine the current indent
                        (thisindent, dummy) = getIndent(text, start=taglinestart, stop=cursor)
                        #lenthisindent = len(thisindent)
                        #print levelindent, "thisindent", repr(thisindent)
                        if thisindent is None:
                            raise ValueError("how did I find a tag on an all white line?")
                        # skip to the end of the indented region, or to end of text segment to scan
                        laststartpoint = endofregion = cursor
                        donescanning = 0
                        while endofregion<end and not donescanning:
                            (nextindent, endofline) = getIndent(text, start=endofregion, stop=end)
                            #print levelindent, "nextindent is", repr(nextindent)
                            laststartpoint = endofregion
                            endofregion = endofline+1
                            if nextindent==thisindent:
                                # done scanning, found same indent
                                #print levelindent, "indent match found", repr(text[laststartpoint:endofline])
                                donescanning = 1
                            elif nextindent is None:
                                # all white line: keep looking
                                pass
                            else:
                                pass
                                # check that indentation matches (THIS IS WRONG IN CASE OF TWO DEDENTS)
##                                if nextindent[:lenthisindent]!=thisindent:
##                                    print "<h1>SEGMENT::</H1><pre>"+text[cursor:endofline]+"</pre>"
##                                    raise ValueError, "bad indent following %s in %s" % (
##                                        repr( text[taglinestart:cursor]),
##                                        repr( text[laststartpoint:endofline]) )
                                #print levelindent, "deeper indent found", repr(text[laststartpoint:endofline])
                                # otherwise the indentation must be deeper, keep looking
                        if endofregion>=end and not donescanning:
                            # consume the whole buffer
                            laststartpoint = end
                        # translate the region
                        xmlcontent = self.text2XML(text, start=cursor, end=laststartpoint, level=level+1)
                        cursor = laststartpoint#endofregion+1
                    # emit xml content
                    #print levelindent, "xmlcontent is", repr(xmlcontent)
                    resultlist.append(xmlcontent)
                    # emit the close tag
                    resultlist.append("</%s> " % (tagname))
        #print levelindent, "resultlist is", resultlist
        xml = "".join(resultlist)
        return xml
    def XML2text(self, xml):
        from reportlab.lib import rparsexml
        xml = textEscape(xml)
        parsedxml = rparsexml.parsexmlSimple(xml)
##        from pprint import pprint
##        print "===INPUT=="; print xml; print "===END OF INPUT==="
##        print "===PPRINT OF PARSE=="; pprint(parsedxml); print "===END OF PPRINT==="
        text = self.parsedXML2text(parsedxml, indent=self.indent)
        return text
    def parsedXML2text(self, parsedxml, indent=None, level=0):
        (name, attdict, textlist, extra) = parsedxml
##        print "   "*level, "TAG=", repr(name), attdict
##        print "   "*level, "CONTENT=",
##        if not textlist:
##            print "   "*level, repr(textlist)
##        else:
##            print
##            for x in textlist:
##                print "   "*level, "=", repr(x)
        # translate the name, if defined
        originalname = name
        name = self.TagToAbbreviation.get(name, name)
        # determine the attribute string
        attributestring = ""
        if attdict:
            attlist = []
            attnames = list(attdict.keys())
            attnames.sort()
            for attname in attnames:
                attvalue = attdict[attname]
                if '"' in attvalue:
                    raise ValueError("this translator does not support double quotes in attribute values "+repr((attname,attvalue)))
                formatted = '%s="%s"' % (attname, attvalue)
                attlist.append(formatted)
            attributestring = " ".join(attlist)
            attributestring = "(%s)" % attributestring
        if indent is None:
            indent = self.indent
        if textlist is None:
            # "empty" tag -- if contentless, omit the !
            if not name:
                raise ValueError("empty tag with no name makes no sense!")
            if self.contentlessTags.get(originalname, None):
                return ".%s%s\n" % (name, attributestring)
            else:
                return ".%s%s!\n" % (name, attributestring)
        else:
            # tag with content
            # process the content list
            processedtextlist = []
            cursor = 0
            nlist = len(textlist)
            while cursor<nlist:
                e = textlist[cursor]
                cursor = cursor+1
                if isinstance(e,str):
                    processed = unindentText(e)
                elif isinstance(e,tuple):
                    processed = self.parsedXML2text(e, indent, level=level+1)
                    # if the next element is a string, trim at most one newline from white prefix (prettification)
                    if cursor<nlist:
                        nexte = textlist[cursor]
                        if nexte and isinstance(nexte,str):
                            ln = nexte.split("\n")
                            if not ln[0].strip():
                                textlist[cursor] = "\n".join(ln[1:])
                else:
                    raise ValueError("I don't know what to do with %r in parsed text list" % e)
##                print "   "*level, "before===", repr(e)
##                print "   "*level, "processed", repr(processed)
                processedtextlist.append(processed)
            processedtext = "".join(processedtextlist) # newlines handled lower down
            # if the tag is "" then just return the processed text
            if not name:
##                print; print "   "*level, repr(processedtext); print
                return processedtext
            # use the indent convention if there is a newline not at the end of text or if text is long
            inlineConvention = ".%s%s %s\n" % (name, attributestring, processedtext)
            inlinelen = len(inlineConvention)
            if inlinelen>80 or "\n" in processedtext.rstrip():
                # indent convention
                indentedtext = indentText(processedtext)
                # trim extra leading newline if present
                if indentedtext and indentedtext[0]=="\n": indentedtext=indentedtext[1:]
                if indentedtext and indentedtext[-1]=="\n": indentedtext=indentedtext[:-1]
                result = ".%s%s\n%s\n" % (name, attributestring, indentedtext)
##                print; print "   "*level, repr(result); print
                return result
            else:
                # inline convention
                result = inlineConvention
                if inlinelen>50:
                    # prepend newline
                    result = "\n" + result
##                print "   "*level, repr(result)
                return result
            raise ValueError("unreachable code")

def xmlEscape(text):
    text = text.replace("&", "&amp;")
    text = text.replace("<", "&lt;")
    text = text.replace(">", "&gt;")
    text = replacemarks(text, "**", "<b>", "</b>")
    text = replacemarks(text, "*", "<i>", "</i>")
    ### remove the "." escape hack
    text = text.replace("\\.", ".")
    return text

def replacemarks(text, marker, starttag, endtag):
    stext = text.split(marker)
    nsegs = len(stext)
    if nsegs!=0 and nsegs % 2 != 1:
        raise ValueError("odd number of %s markers in text" % repr(marker))
    if len(stext)<2:
        # no marker found
        return text
    first = stext[0]
    # consume extra space at end of prefix
    if first[-2:]=="  ":
        first = first[:-1]
    outlist = [first]
    del stext[0]
    while stext:
        [marked, after] = stext[:2]
        # consume extra spaces in after
        if after[:2]=="  ":
            after = after[1:]
        if after[-2:]=="  ":
            after = after[:-1]
        outlist.append(starttag)
        outlist.append(marked)
        outlist.append(endtag)
        outlist.append(after)
        del stext[:2]
    return "".join(outlist)

def getIndent(text, start=0, stop=None):
    # find the whitespace prefix for the line starting at start
    # if the line is all white, return None (ie, ignore).  Also return the end of line
    stop1 = text.find("\n", start)
    if stop is None:
        stop = stop1
        if stop<start:
            stop = len(text)
    elif stop1>=start and stop>stop1:
        stop = stop1
    line = text[start:stop]
    sline = line.strip()
    if not sline:
        return (None, stop)
    indentend = line.find(sline)
    if indentend<0:
        raise ValueError("CAN'T FIND STRIP STRING IN STRING!")
    thisindent = line[:indentend]
    return (thisindent, stop)

def unindentText(textblock):
    "make sure text block is flush left, also remove whitespace preceding newlines"
    lines = textblock.split("\n")
    slines = [l.strip() for l in lines]
    result = "\n".join(slines)
    return result

def textEscape(xml):
    """ for any '.' that precedes an alpha, change it to a '\.'
        convert <i> and </i> to * and *
        convert <b> and </b> to ** and **
    """
    xml = xml.replace("<i>", " *")
    xml = xml.replace("</i>", "* ")
    xml = xml.replace("<b>", " **")
    xml = xml.replace("</b>", "** ")
    sxml = xml.split(".")
    outlist = [sxml[0]]
    sawbackslash = 0
    for e in sxml[1:]:
        if not sawbackslash and e[0:1] in ascii_letters:
            outlist.append('\\')
        outlist.append('.')
        outlist.append(e)
        if e[-1:]=='\\':
            sawbackslash = 1
        else:
            sawbackslash = 0
    return "".join(outlist)

def indentText(textblock, indentation=" "*4):
    "add indentation to all nonwhite lines of textblock, remove extra whitespace in all white lines"
    lines = textblock.split("\n")
    ilines = []
    for l in lines:
        l2 = l.strip()
        if l2:
            l = indentation+l
            ilines.append(l)
        else:
            ilines.append("") # all white
    # remove extra white lines at the end
    while ilines and not ilines[-1]:
        del ilines[-1]
    return "\n".join(ilines)

XMLSAMPLE = """<this type="xml">text brackets: &lt;&gt;. <b>in</b> <funnytag foo="bar"/> xml</this>
                <narf><narf></narf></narf>
                 <!-- comment -->
                 <tag with="<brackets in values>">just testing brackets feature</tag>"""

XMLSAMPLE2 = '''<endorsementHeader number="{{number}}" title="APPROVED CUSTOMER ENDORSEMENT" issuedBy="{{issuedBy}}" issuedTo="{{companyName}}"
    policyNo="{{policyNo}}" effDate="{{effectiveDate}}"/>


<p>The following are <b>approved customers</b> on the terms outlined below:</p>
<spacer length="0.5cm"/>

<approvedCustomerHeader/>

{{for C in ApprovedCustomers}}

{{script}}
from projects.AIGRisk.do_policy_main import formatLines
theName = formatLines([ C["Name1"], C["Name2"] ], 32)
theAddress = formatLines([ C["Address1"], C["Address2"] ], 32)
amountApproved = C["limitApproved"].strip()[:-3]
currencyApproved = C["limitApproved"].strip()[-3:]
{{endscript}}

<approvedCustomer>
    <ACName><p>{{theName}}</p></ACName>
    <ACAddress><p>{{theAddress}}</p></ACAddress>
    <ACEffDate><p>{{ C["effDate"] }}</p></ACEffDate>
    <ACLimit><p>{{ amountApproved + "\n" + currencyApproved }}</p>
    </ACLimit>
    <ACMaxTerms><p>{{ C["maxPayTerms"] }}</p></ACMaxTerms>
</approvedCustomer>
{{endfor}}

<spacer length="0.5cm"/>
<p>All other terms, conditions, and exclusions of this Policy shall remain unchanged.</p>

<signatures/>

'''

XMLSAMPLE3 = """ <ACLimit><p>{{ amountApproved + "\n" + currencyApproved }}</p>
    </ACLimit> """

XMLSAMPLE4 = """


<!-- declarations page starts here -->

<AIGLogo/>

<bigCentered>{{THECOMPANY}}</bigCentered>

<centered><font face="Helvetica">
(A Stock Insurance Company, Hereinafter Called the Company)</font>
</centered>

<centered>
<b>Executive Offices</b>
</centered>

<centered>
<b>{{issuedByAddress1}}</b>
</centered>

<centered>
<b>{{issuedByAddress2}}</b>
</centered>

<spacer length="0.5cm"/>

<centered>
<b>AIG TRADECREDIT.COM DECLARATIONS</b>
</centered>

<spacer length="0.5cm"/>

<p>If <b>you</b> have any questions about <b>your</b> insurance policy, 
or questions about claims relating to <b>your</b> insurance policy, please contact <b>us</b>
at 1-888-437-3662.</p>

<spacer length="1cm"/>

<p>
POLICY NO.: <font face="Courier">{{policyNo}}</font>
</p>

<spacer length="0.5cm"/>

<policyItem number="Item 1." title="Insured's Name and Address:">
    <p>{{companyName}}</p>
    {{if companyAddress1.strip()}} <p>{{companyAddress1}}</p> {{endif}}
    {{if companyAddress2.strip()}} <p>{{companyAddress2}}</p> {{endif}}
    {{if companyAddress3.strip()}} <p>{{companyAddress3}}</p> {{endif}}
    {{if companyAddress4.strip()}} <p>{{companyAddress4}}</p> {{endif}}
</policyItem>

<policyItem number="Item 1.a." title="Insured's E-Mail Address:">
    <p>{{contactEmail}}</p>
</policyItem>

<policyItem number="Item 2." title="Policy Period:">
    <p>For <b>shipments</b> made on and after <b>
    {{effectiveDate}}
    </b> (12:01 a.m., at the
    address stated in Item 1. of these Declarations) to 
    <b> 
    {{expirationDate}}
    </b> (12:01 a.m., at the address stated in Item 1. of these
    Declarations)</p>
</policyItem>

<policyItem number="Item 3." title="Premium:"><p>
    As per the attached Premium Endorsement
</p></policyItem>

<policyItem number="Item 4." title="Insured Percentage:"><p>
For <b>documented obligations</b> due and owing from an <b>approved
customer</b> which are submitted to the Collection Agency stated in Item 6.
of these Declarations:
</p></policyItem>

<policyItem2 number="">
<p>a. Within 0 to 90 days from the oldest unpaid due date of payment: 85% of the <b>approved customer limit.</b></p>
<p>b. Within 91 to 120 days from the oldest unpaid due date of payment: 70% of the <b>approved customer limit.</b></p>
<p>c. On or after 121 days from the oldest unpaid due date of payment: 0% of the <b>approved customer limit.</b></p>
</policyItem2>

<policyItem number="Item 5." title="Deductible:"><p>
{{deductible}} for <b>shipments</b> made during the period of {{effectiveDate}} to {{expirationDate}}.
</p></policyItem>

<policyItem3 number="Item 6." title="Collection Agency's Name and Address">
    <p>{{collectionAgencyName1}}</p>
    {{if collectionAgencyName2.strip()}} <p>{{collectionAgencyName2}}</p> {{endif}}
    {{if collectionAgencyAddress1.strip()}} <p>{{collectionAgencyAddress1}}</p> {{endif}}
    {{if collectionAgencyAddress2.strip()}} <p>{{collectionAgencyAddress2}}</p> {{endif}}
    {{if collectionAgencyAddress3.strip()}} <p>{{collectionAgencyAddress3}}</p> {{endif}}
    {{if collectionAgencyAddress4.strip()}} <p>{{collectionAgencyAddress4}}</p> {{endif}}
</policyItem3>

<policyItem number="Item 7." title="Insured Product(s):">
<p>{{insuredProduct}}</p>
</policyItem>

<policyItem2 number="Item 8."><p>
These Declarations and the following Endorsements are made part of this policy: </p>
<p> {{endorsementList1}}
    {{endorsementList2}}
</p>
</policyItem2>

<p>
<b>IN WITNESS WHEREOF, we</b> have caused this Policy and this 
Declarations page to be signed by <b>our</b> President, <b>our</b> Secretary and
<b>our</b> duly authorized representative.
</p>

<signatures/>

<!-- declarations page ends here -->
"""

XMLSAMPLE5 = """
<endorsementHeader number="{{number}}" title="VIRGINIA AMENDATORY ENDORSEMENT" issuedBy="{{issuedBy}}" issuedTo="{{companyName}}"
    policyNo="{{policyNo}}" effDate="{{effectiveDate}}"/>

<spacer length="0.5cm"/>

<p>The Policy is amended as follows:</p>

<spacer length="0.5cm"/>

<p>The first paragraph of <b>ARTICLE IV., HOW A CLAIM IS PAID,</b> Paragraph
<b>B., CALCULATION OF YOUR CLAIM</b>
is deleted in its entirety and replaced with the following:</p>

<spacer length="0.5cm"/>

<policyItem2 number = "B.">
<p>CALCULATION OF <b>YOUR</b> CLAIM</p>

<spacer length="0.5cm"/>

<p><b>We</b> shall pay <b>you</b>
the lesser of the following
<b>A.</b> the Insured Percentage of the
<b>approved customer's uncollected debt,</b>
minus the amount of the Deductible, if any, stated in the Declarations, or
<b>B.</b> the Insured Percentage of the
<b>approved customer's limit,</b>
minus the amount of the Deductible, if any, stated
in the Declarations.
</p>
</policyItem2>

<spacer length="0.5cm"/>

<p>In <b>ARTICLE VI., CONDITIONS,</b>
Paragraph G., CONFORMANCE TO STATUTE is deleted in its entirety.</p>

<spacer length="0.5cm"/>

<p>In <b>ARTICLE VI., CONDITIONS,</b>
Paragraph I., ARBITRATION is deleted in its entirety and replaced with the following:</p>

<spacer length="0.5cm"/>

<policyItem2 number = "I.">
<p>ARBITRATION</p>

<spacer length="0.5cm"/>

<p><b>You</b> must inform <b>us</b> in writing of any dispute you may have regarding coverage 
under this Policy within two (2) years of an <b>uncollected debt.</b>
All disputes, including disputes over coverage, will be submitted to
the American Arbitration Association for resolution.  The dispute will be
handled in accordance with their prevailing rules for commercial disputes. Their decision however,
shall be non-binding on <b>you</b> and on <b>us.</b>
</p>
</policyItem2>

<spacer length="0.5cm"/>
<p>In <b>ARTICLE VI., CONDITIONS,</b>
Paragraph L., REPRESENTATIONS is deleted in its entirety and replaced with the following.</p>

<spacer length="0.5cm"/>

<policyItem2 number = "L.">
<p>CONDITIONS</p>

<spacer length="0.5cm"/>

<p>By accepting this Policy, <b>you</b> agree that the statements made
in the Application and Declarations are true and <b>you</b> also agree
that this Policy, which includes the Application, is issued
in reliance upon the truth of those representations.
Accordingly, this Policy may be voided by <b>us</b> in any case of
fraud, intentional concealment, or misrepresentation of
material fact by <b>you</b>.
</p>
</policyItem2>

<spacer length="0.5cm"/>
<p>All other terms, conditions, and exclusions shall remain unchanged.</p>

<spacer length="0.5cm"/>

<signatures/>

"""

#XMLSAMPLE = XMLSAMPLE5 + XMLSAMPLE4 + XMLSAMPLE3 + XMLSAMPLE2

XMLSAMPLE = XMLSAMPLE2

def example0():
    c = XMLTextConverter()
    c.abbreviate("AIGLogo", "logo")
    c.abbreviate("centered", "c")
    c.abbreviate("bigcentered", "B")
    c.abbreviate("spacer", "s")
    c.abbreviate("signatures", "sign")
    c.abbreviate("Alphalist", "A")
    c.abbreviate("alphalist", "a")
    c.abbreviate("numberlist", "n")
    c.abbreviate("policyItem", "Item")
    c.abbreviate("policyItem2", "N")
    c.abbreviate("element", "li")
    c.contentless("signatures")
    c.contentless("spacer")
    return c
    
def test():
    c = example0()
##    c = XMLTextConverter()
##    c.abbreviate("this", "t")
##    c.abbreviate("funnytag", "f")
##    c.abbreviate("tag", "tg")
    xml = savexml = XMLSAMPLE
    savetext = ""
    print("=== XML at start ===")
    print(xml)
    for i in range(20):
        print("--------------- PASS", i, "--------------")
        text = c.XML2text(xml)
        print("=== TEXT ===", end=' ')
        if text==savetext:
            print("unchanged!")
        else:
            print("changed!")
        # try to analyse the text changes
        stext = text.split("\n")
        ssavetext = savetext.split("\n")
        if len(stext)!=len(ssavetext):
            print("DIFFERING NUMBERS OF LINES -- OLD", len(ssavetext), "NEW", len(stext))
        for (a,b) in map(None, stext, ssavetext):
            if a!=b:
                print("FIRST LINES THAT DIFFER")
                print("NEW:", repr(a))
                print("OLD:", repr(b))
                print("===============")
                break
        print(text)
        xml = c.text2XML(text)
        print("=== XML ===", end=' ')
        if xml==savexml:
            print("unchanged!")
        else:
            print("changed!")
        print(xml)
        savexml = xml
        savetext = text

if __name__=="__main__":
    test()




__code__='Gas1]gMYb8&=k*r=]X<G;;ANmPD7>Y[0i/@V$YkM-lOL+@=VC<.#A>."[R4\\2IhVTQ7H/dS0nR*97NbPdEg`ds.W;[TuC,*$;8!`eaCXA1S[,1&\'TI)denr6p,<l%%O6Lr;#g[1#.@#+N!)ZP8&1&I+!;"X)CD*\\=9GW,/c[d0(BL"UB)n7u56*eRGRgq]GTO\'uGQDb$n3b!LJ0+f@%!W*J\'>RBE0*>M]r!`\'EZ(-/L1=sG$0>h_HD8E.TZHa=Y$Y0^,9FrlWKT5.14=>:kn8h?I6oeM)@HU*\'2M1`+c)O%3<0fsL=X0q3bj\'F)?tAiS@E28t]EWpI$asZR`rN0q+T[?"0"s),"!#`C)2*%QqOg9bA!U84!tGc2+&oCpRs?8E&3j#$nh].IbZfS03dB)&puk;i//G[jp9dr?68i&t;AI3_@hi#kUTHl$KfRGe8&f0=(T7Nao^@ZpjA[Z0latG*kIYW/K\'mHQ44.K$FXNYMKMHhi(hq-8AK1oK.GCLBEbiQa;o3Ht?L3`Lk7p_;)^dV%GeN0`0.23jI%m&D=-Cjs75ipXK;$sLM37UA36^"@a"gARG5Y-^h0h>k_p%iA-k\'XV8[MDb,_<CfXZ=-:XjcaNh>R;#8!TR+^ZbmCfWba(i9!dac]9Ee%[5O?PV&eIW!FnEdY##[R#lnN$?Ga\'?D>ppeJ7oErk7*ir(luibHJZZi1<mcB;Um2<g\'ERZ0RjJ?^j]9LADr><OeXXkp<V.W@oX>/+9m@6PO]);8J9b8V[B!#,AMKDV`#qj8SFmhqZ!D/T3htgF"DMZ\'Hm5,>+p7CG5]3*:q8Y%4C,[qVu5^Ce6<D2fAj]1>!&rmAbfGmIp-pe^*N1\\=!r?gO#/<^ZdQ[g09"C8`ZOOnn*$/m.*;bQFDW)R&Eh\'f[%D[.U2O7^jNEPQ2VaYdk=iu01>V"]E2K#Ofd]c!]0%gBYm4f1,5.fRdW1iJYlYqH<,Dj4/*;8GHPs`pI`D1@7%"gTd<Zr+,,_\'OZJJ2_=RRQ,,c.-@bd[nC,sQHZ!3#I`A/rrr_G+7j+h,mrQ*sN=\'VK0:gXopU418ZP-HS\'`db]rA;1~>'



__code__='Gas1]gMYb8&=k*r=]Sct;;Cf01i+eHoh[-)->8=3BV7>%!*V7S,VGH""O1*(N%\'omY&\'MFSfFCVrK7(K5D-0<M`u9<(1T3N,4#e?]//Wo%/9,G0_`1Qo*:`[?Y&XUGQ9t&!!INMCl]%p"@rR6^l.IG+:URS-D1Op!,M]h!4DkD!S`-;!1*]C"\'c$m$ePJB#M92B&%D]952ZU[%fefsr!;e%^eXq`!aYs+\'_!l_S:,91fDXSZ_QUmpJ_\'nTj:%gfZ2qumL_kQVjtdC:&\\AMI)5las%Td&f]2jg6E5"TD/s%TK<=r2r7hph_*NoTj-N>+Z/1&AQ(n.d=Me7V(J,rpCOFSPV(LEQ\'!KLi\\$plu95CjUCN`fqm%6-*c]2uTODW\\2(O]7kt[UF0q^Rk<ANuecQYMF]t\\7_*84&Q;eX)]AU/K1%7\'&Bld/!odL.?`+!k<#9?Q2Crg761=QWa\\!=.erSCF3"*RT<#1\\gZ_LU\'^,G:<Z7`s*L90\\EG7FgJS9tEk_R+0FL$kp>J?^4A*Nl%g\\8`;:1@N_q>Q:D&>,Jk2T8[lF(UODpW[)Z2G:nCmu9fU])>03WR>[gAT,(S7Wt$NJK77S9QGu5VMHMIG.Kj"\\#XZFD81R>E8[[01Cgj`0HE55@1M,1=kaJH8YDh8Yu>5WBbi/7$IeA2^HbI`5o\'X7`TeRGC6IhMr8%@)_QNjpl*ju(IA=T+WH-9XVl?dD;<\\bGFgnk9QhV!^/PrHdUUW0P\'-0nV:6/K6C.F=5ltiE5h&r$o"g/BLj]aZ]hOnl_k+/^DhYrs&T+p*[P_Y*].LBQVB^M-PnTctRP>`Ap_O^?IaR.ZD?j:Mr$$Tsj(F;.c#AMpu/%TjflgJdU-ZjS%TNHKqBb%MZK.*&,5sH1QqC(1[Hh+Q-Nat+HOZp.E7VQ,8kd?0@@Pqp-3b5*pYb7SGlA9`l^bR-=/!;c%ckFf]bt;MU3)_]lDi@"W\\!Ua/pnIIM.Ah#gjg+,PMS$f[<g?!AW"-QM4]*Bq>31A)#B<F6\'sbfgAL#WDS<m]g\'`3c!<X&Go>0>E?-i3\\2_t.Fl;4MPW$F>qS(""l;"\\iN<hZ~>'
